#include <string>

class IWindow
{
public:
	virtual void show (std::string&) = 0;
	virtual ~IWindow() {}
};

class WindowImpl : public IWindow
{
public:
	void show (std::string &) { /* ... */ }
};

class WinDecorator : public IWindow
{
protected:
	IWindow& m_win;
public:
	WinDecorator(IWindow& win): m_win(win) {}
	void show (std::string &) = 0;
};

class ScrollBar : public WinDecorator
{
public:
	void scroll (int) { /* ... */ }
	IWindow& noScrollBar() { return m_win; }
	ScrollBar (IWindow& win) : WinDecorator(win) {}
	void show (std::string &) { /* ... */ }
};

//.cpp:
void WinDecorator::show(std::string& text) { m_win.show(text); }

int main ()
{
	WindowImpl wi;
	IWindow& w = wi;
	w = ScrollBar(w);
	w = static_cast<ScrollBar&>(w).noScrollBar();
}
