#include <memory>
#include <string>
#include <map>

class DoesNotExist : public std::exception {};

template <typename Prototype>
class PrototypeManager
{
	typedef std::map <std::string, Prototype*> map;
	map m_table;
public:
	void insert (std::string const& name, Prototype const& obj)
	{
		m_table[name] = obj.clone();
	}
	std::auto_ptr<Prototype> create (std::string const& name)
	{
		Prototype* proto = m_table[name];
		if (!proto) throw DoesNotExist();
		std::auto_ptr<Prototype> ret(proto->clone());
		return ret;
	}
	~PrototypeManager()
	{
		for (typename map::const_iterator it = m_table.begin();
				it!=m_table.end(); ++it)
		{
			delete it->second;
		}
	}
};
